//---------------------------------------------------------------------------
/**
 *  @file AppDlg.cpp
 *
 *  $Id: AppDlg.cpp,v 1.3 2001/05/18 11:07:00 chris Exp $
 *
 *  @author Christopher Kohlhoff <chris@kohlhoff.com>
 *
 *  This file contains the dialog used to create an ACE project.
 */
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Registry.hpp>
#include <FileCtrl.hpp>
#include <memory>
#include "AppDlg.h"
#include "ProjectCreator.h"
#include "FileGenerator.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TAppDialog *AppDialog;
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x560)
const char* sRegKey = "Software\\Borland\\C++Builder\\6.0\\ACETAO";
#elif (__BORLANDC__ >= 0x550)
const char* sRegKey = "Software\\Borland\\C++Builder\\5.0\\ACETAO";
#elif (__BORLANDC__ >= 0x540)
const char* sRegKey = "Software\\Borland\\C++Builder\\4.0\\ACETAO";
#else
#error Unsupported version of C++Builder
#endif
//---------------------------------------------------------------------------
__fastcall TAppDialog::TAppDialog(TComponent* Owner, String prefix)
  : TForm(Owner),
    FPrefix(prefix)
{
  // First try reading ACE_ROOT and its type from the registry
  std::auto_ptr<TRegistry> reg(new TRegistry);
  reg->OpenKeyReadOnly(sRegKey);
  String location = reg->ReadString("Location");
  int locationType = 0;
  try { locationType = reg->ReadInteger("LocationType"); } catch (...) {}

  // Otherwise just use a reasonable default value
  if (location.Length() == 0)
    location = "C:\\ACETAO";

  ACELocation->Text = location;
  LocationType->ItemIndex = locationType;

  AppTypeRadioClick(0);
  UsePCHClick(0);
}
//---------------------------------------------------------------------------
void __fastcall TAppDialog::AppTypeRadioClick(TObject *Sender)
{
  UseVCL->Enabled = Console->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TAppDialog::OKButtonClick(TObject *Sender)
{
  // Save the new value for ACE_ROOT
  std::auto_ptr<TRegistry> reg(new TRegistry);
  reg->OpenKey(sRegKey, true);
  reg->WriteString("Location", ACELocation->Text);
  reg->WriteInteger("LocationType", LocationType->ItemIndex);

  if (ToolServices && ToolServices->CloseProject())
  {
    Visible = false;

    // Create a new file generator for this project.
    std::auto_ptr<TFileGenerator> generator(new TFileGenerator);

    // All projects are created with the source code visible.
    TCreateProjectFlags projectFlags;
    projectFlags << cpCanShowSource;
    String deco = "_b";
    if (VCL->Checked)
    {
      generator->SetValue("APPTYPE", "VCL");
      generator->SetValue("PASCAL", "1");
      projectFlags << cpApplication;
#if defined(__BORLANDC__) && (__BORLANDC__ < 0x550)
      // We only need to use separate VCL-compatible libraries for BCB 4.
      deco += "p";
#endif
    }
    else if (WindowsAPI->Checked)
    {
      generator->SetValue("APPTYPE", "Window");
      generator->SetValue("PASCAL", "");
      projectFlags << cpCustom;
    }
    else
    {
      generator->SetValue("APPTYPE", "Console");
      generator->SetValue("PASCAL", UseVCL->Checked ? "1" : "");
      projectFlags << cpConsole;
#if defined(__BORLANDC__) && (__BORLANDC__ < 0x550)
      // We only need to use separate VCL-compatible libraries for BCB 4.
      if (UseVCL->Checked)
        deco += "p";
#endif
    }

    generator->SetValue("TYPE", FPrefix);
    generator->SetValue("ACELOCATION", ACELocation->Text);
    generator->SetValue("LOCATIONTYPE",
        LocationType->ItemIndex == 0 ? "Source" : "Install");
    generator->SetValue("USEPCH", UsePCH->Checked ? "1" : "");
    generator->SetValue("PCHFILE", PCHCsmFile->Text);
    if (Static->Checked)
    {
      generator->SetValue("STATIC", "1");
      deco += "s";
    }
    else
    {
      generator->SetValue("STATIC", "");
    }
    if (DebugLibs->Checked)
    {
      generator->SetValue("DEBUG", "1");
      deco += "d";
    }
    else
    {
      generator->SetValue("DEBUG", "");
    }

    String libs = "ACE" + deco + ".lib";
    if (FPrefix == "TAO")
      libs += " TAO" + deco + ".lib";
    for (int i = 0; i < CORBALibs->Items->Count; ++i)
      if (CORBALibs->Checked[i])
        libs += " TAO_" + CORBALibs->Items->Strings[i] + deco + ".lib";
    generator->SetValue("LIBRARIES", libs);

    ToolServices->ProjectCreateEx(new TProjectCreator(ToolServices,
          generator.get()), projectFlags);
    generator.release();
  }
}
//---------------------------------------------------------------------------
void __fastcall TAppDialog::SelectDirectoryButtonClick(TObject *Sender)
{
  String dir = ACELocation->Text;
  if (SelectDirectory("Select ACE+TAO directory:", "", dir))
    ACELocation->Text = dir;
}
//---------------------------------------------------------------------------
void __fastcall TAppDialog::UsePCHClick(TObject *Sender)
{
  PCHCsmFile->Enabled = UsePCH->Checked;
}
//---------------------------------------------------------------------------
