//---------------------------------------------------------------------------
/**
 *  @file FileGenerator.h
 *
 *  $Id: FileGenerator.h,v 1.2 2001/05/18 11:07:01 chris Exp $
 *
 *  @author Christopher Kohlhoff <chris@kohlhoff.com>
 *
 *  This file contains a class for generating project and source files.
 */
//---------------------------------------------------------------------------
#ifndef FileGeneratorH
#define FileGeneratorH
//---------------------------------------------------------------------------
#include <System.hpp>
#include <map>
//---------------------------------------------------------------------------
class TFileGenerator
{
public:
  // Constructor.
  TFileGenerator();

  // Destructor.
  ~TFileGenerator();

  // Set the value of a variable.
  void SetValue(const String& key, const String& value);

  // Get the value of a variable.
  String GetValue(const String& key) const;

  // Generate a string of output using the given template file.
  String Generate(const String& templateName) const;

private:
  std::map<String, String> FValues;

  // Get the text of a file after replacing any macro values
  void ExpandConditionals(TStrings* strings) const;
  int ExpandLiveConditionalBranch(TStrings* strings, int startRow) const;
  int ExpandDeadConditionalBranch(TStrings* strings, int startRow) const;
  bool ConditionalIsTrue(String conditional) const;
  bool IsIfLine(String line) const;
  bool IsElseLine(String line) const;
  bool IsEndifLine(String line) const;
  void ExpandMacros(TStrings* strings) const;
  String ExpandMacros(String line) const;
};
//---------------------------------------------------------------------------
#endif
