//---------------------------------------------------------------------------
/**
 *  @file ProjectCreator.cpp
 *
 *  $Id: ProjectCreator.cpp,v 1.3 2001/05/24 11:00:49 chris Exp $
 *
 *  @author Christopher Kohlhoff <chris@kohlhoff.com>
 *
 *  This file contains class that is used by the IDE to create projects.
 */
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Istreams.hpp>
#include <memory>
#include "ProjectCreator.h"
#include "FileGenerator.h"
//---------------------------------------------------------------------------
__fastcall TProjectCreator::TProjectCreator(TIToolServices* toolServices,
    TFileGenerator* generator)
: FAppType(generator->GetValue("APPTYPE")),
  FToolServices(toolServices),
  FGenerator(generator)
{
}
//---------------------------------------------------------------------------
__fastcall TProjectCreator::~TProjectCreator()
{
}
//---------------------------------------------------------------------------
bool __stdcall TProjectCreator::Existing(void)
{
  return false;
}
//---------------------------------------------------------------------------
String __stdcall TProjectCreator::GetFileName(void)
{
  return "";
}
//---------------------------------------------------------------------------
String __stdcall TProjectCreator::GetFileSystem(void)
{
  return "";
}
//---------------------------------------------------------------------------
String __stdcall TProjectCreator::NewProjectSource(
  const String ProjectName)
{
  return FGenerator->Generate(FAppType + "_ProjectSource");
}
//---------------------------------------------------------------------------
void __stdcall TProjectCreator::NewDefaultModule(void)
{
  std::auto_ptr<TIStreamAdapter> srcStrm;
  std::auto_ptr<TIStreamAdapter> hdrStrm;
  std::auto_ptr<TIStreamAdapter> formStrm;
  IStream* srcIstrm = 0;
  IStream* hdrIstrm = 0;
  IStream* formIstrm = 0;

  String unitName, fileName;
  if (!FToolServices->GetNewModuleName(unitName, fileName))
    return;

  try
  {
    TStringStream* sstrm = new TStringStream(FGenerator->Generate(
          FAppType + "_UnitSource"));
    srcStrm.reset(new TIStreamAdapter(sstrm, soOwned));
    srcIstrm = *srcStrm;
  }
  catch (...)
  {
  }

  try
  {
    TStringStream* sstrm = new TStringStream(FGenerator->Generate(
          FAppType + "_UnitHeader"));
    hdrStrm.reset(new TIStreamAdapter(sstrm, soOwned));
    hdrIstrm = *hdrStrm;
  }
  catch (...)
  {
  }

  try
  {
#if (__BORLANDC__ >= 0x550)
    TStringStream* sstrm = new TStringStream(FGenerator->Generate(
          FAppType + "_UnitForm"));
    formStrm.reset(new TIStreamAdapter(sstrm, soOwned));
#else
    // C++Builder 4 only supports binary form files, so we can't use the
    // generator.
    char module[MAX_PATH];
    GetModuleFileName(HInstance, module, MAX_PATH);
    String filename = ExtractFilePath(module) + FAppType + "_UnitForm.txt";
    TFileStream* fstrm = new TFileStream(filename, fmOpenRead);
    formStrm.reset(new TIStreamAdapter(fstrm, soOwned));
#endif
    formIstrm = *formStrm;
  }
  catch (...)
  {
  }

  TCreateModuleFlags flags;
  flags << cmAddToProject << cmShowSource << cmUnNamed << cmMarkModified;
  if (formIstrm)
    flags << cmShowForm;
  if (hdrIstrm || srcIstrm || formIstrm)
  {
    FToolServices->CreateCppModule(fileName, "", "", "", hdrIstrm, srcIstrm,
        formIstrm, flags);
  }

  srcStrm.release();
  hdrStrm.release();
  formStrm.release();
}
//---------------------------------------------------------------------------
void __stdcall TProjectCreator::NewProjectResource(TIModuleInterface* Module)
{
}
//---------------------------------------------------------------------------
String __stdcall TProjectCreator::GetOptionName(void)
{
  return "";
}
//---------------------------------------------------------------------------
String __stdcall TProjectCreator::NewOptionSource(const String ProjectName)
{
  FGenerator->SetValue("PROJECT", ProjectName);
  return FGenerator->Generate(FAppType + "_Project");
}
//---------------------------------------------------------------------------
