//---------------------------------------------------------------------------
/**
 *  @file SimpleClassExpert.cpp
 *
 *  $Id: SimpleClassExpert.cpp,v 1.2 2001/05/18 11:07:01 chris Exp $
 *
 *  @author Christopher Kohlhoff <chris@kohlhoff.com>
 *
 *  This file contains the expert used for generating all simple classes.
 */
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <memory>
#include <ExptIntf.hpp>
#include <Istreams.hpp>
#include "SimpleClassExpert.h"
#include "SimpleClassDlg.h"
#include "FileGenerator.h"
//---------------------------------------------------------------------------
TSimpleClassExpert::TSimpleClassExpert(const String& prefix,
    const String& name, const String& description)
: FPrefix(prefix),
  FName(name),
  FDescription(description)
{
}
//---------------------------------------------------------------------------
String __stdcall TSimpleClassExpert::GetName()
{
  return FName;
}
//---------------------------------------------------------------------------
String __stdcall TSimpleClassExpert::GetComment()
{
  return FDescription;
}
//---------------------------------------------------------------------------
HICON __stdcall TSimpleClassExpert::GetGlyph()
{
  try
  {
    return LoadIcon(HInstance, FPrefix.c_str());
  }
  catch (...)
  {
    ToolServices->RaiseException(ReleaseException());
    return 0;
  }
}
//---------------------------------------------------------------------------
TExpertStyle __stdcall TSimpleClassExpert::GetStyle()
{
  return esForm;
}
//---------------------------------------------------------------------------
TExpertState __stdcall TSimpleClassExpert::GetState()
{
  return (TExpertState() << esEnabled);
}
//---------------------------------------------------------------------------
String __stdcall TSimpleClassExpert::GetIDString()
{
  return "ACETAO." + FPrefix + ".Expert";
}
//---------------------------------------------------------------------------
void __stdcall TSimpleClassExpert::Execute()
{
  std::auto_ptr<TSimpleClassDialog> dlg(new TSimpleClassDialog(Application));
  dlg->Caption = "New " + FName;
  if (!dlg->Execute())
    return;

  std::auto_ptr<TIStreamAdapter> srcStrm;
  std::auto_ptr<TIStreamAdapter> hdrStrm;
  IStream* srcIstrm = 0;
  IStream* hdrIstrm = 0;

  String unitName, fileName;
  if (!ToolServices->GetNewModuleName(unitName, fileName))
    return;

  TFileGenerator generator;
  generator.SetValue("UNIT", unitName);
  generator.SetValue("CLASS", dlg->ClassName);

  try
  {
    TStringStream* sstrm = new TStringStream(generator.Generate(
          FPrefix + "_Source"));
    srcStrm.reset(new TIStreamAdapter(sstrm, soOwned));
    srcIstrm = *srcStrm;
  }
  catch (...)
  {
  }

  try
  {
    TStringStream* sstrm = new TStringStream(generator.Generate(
          FPrefix + "_Header"));
    hdrStrm.reset(new TIStreamAdapter(sstrm, soOwned));
    hdrIstrm = *hdrStrm;
  }
  catch (...)
  {
  }

  TCreateModuleFlags flags;
  flags << cmAddToProject << cmShowSource << cmUnNamed << cmMarkModified;
  ToolServices->CreateCppModule(fileName, "", "", "", hdrIstrm, srcIstrm,
    0, flags);

  srcStrm.release();
  hdrStrm.release();
}
//---------------------------------------------------------------------------
